##############################################################################
#
# Copyright (c) 2007 Nexedi SA and Contributors. 
# All Rights Reserved.
#          Romain Courteaud <romain@nexedi.com>
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
##############################################################################

import unittest

from Products.ERP5Type.tests.ERP5TypeTestCase import ERP5TypeTestCase
from AccessControl.SecurityManagement import newSecurityManager
from Products.ERP5Type.tests.Sequence import SequenceList
from zExceptions import BadRequest
from Products.ERP5Type.Tool.ClassTool import _aq_reset
from Testing.ZopeTestCase.PortalTestCase import PortalTestCase

class TestWorklist(ERP5TypeTestCase):

  run_all_test = 1
  quiet = 1
  login = PortalTestCase.login

  checked_portal_type = 'Organisation'
  checked_validation_state = 'draft'
  not_checked_validation_state = 'not_draft'
  checked_workflow = 'validation_workflow'
  worklist_assignor_id = 'assignor_worklist'
  actbox_assignor_name = 'assignor_todo'
  worklist_owner_id = 'owner_worklist'
  actbox_owner_name = 'owner_todo'
  worklist_assignor_owner_id = 'assignor_owner_worklist'
  actbox_assignor_owner_name = 'assignor_owner_todo'
  worklist_desactivated_id = '%s_desactivated' % worklist_owner_id
  actbox_desactivated_by_expression = '%s_desactivated' % actbox_owner_name
  worklist_wrong_state_id = '%s_wrong_state' % worklist_owner_id
  actbox_wrong_state = '%s_wrong_state' % actbox_owner_name

  worklist_int_variable_id = 'int_value_workflist'
  actbox_int_variable_name = 'int_value_todo'
  int_catalogued_variable_id = 'int_index'
  int_value = 1

  def getTitle(self):
    return "Worklist"

  def getBusinessTemplateList(self):
    """
    Return list of bt5 to install
    """
    return ('erp5_base',)

  def getUserFolder(self):
    """
    Return the user folder
    """
    return getattr(self.getPortal(), 'acl_users', None)

  def createManagerAndLogin(self):
    """
    Create a simple user in user_folder with manager rights.
    This user will be used to initialize data in the method afterSetup
    """
    self.getUserFolder()._doAddUser('manager', '', ['Manager'], [])
    self.login('manager')

  def createERP5Users(self, user_dict):
    """
    Create all ERP5 users needed for the test.
    ERP5 user = Person object + Assignment object in erp5 person_module.
    """
    portal = self.getPortal()
    module = portal.getDefaultModule("Person")
    # Create the Person.
    for user_login, user_data in user_dict.items():
      # Create the Person.
      self.logMessage("Create user: %s" % user_login)
      person = module.newContent(
        portal_type='Person', 
        reference=user_login, 
        password='hackme',
      )
      # Create the Assignment.
      assignment = person.newContent( 
        portal_type = 'Assignment',
        group = "%s" % user_data[0],
        function = "%s" % user_data[1],
        start_date = '01/01/1900',
        stop_date = '01/01/2900',
      )
      assignment.open()
    # Reindexing is required for the security to work
    get_transaction().commit()
    self.tic()

  def createUsers(self):
    """
    Create all users needed for the test
    """
    self.createERP5Users(self.getUserDict())

  def getUserDict(self):
    """
    Return dict of users needed for the test
    """
    user_dict = {
      'foo': [None, None],
      'bar': [None, None],
    }
    return user_dict

  def stepLoginAsFoo(self, sequence=None, sequence_list=None, **kw):
    self.login("foo")

  def stepLoginAsBar(self, sequence=None,
                       sequence_list=None, **kw):
    self.login("bar")

  def createDocument(self):
    module = self.getPortal().getDefaultModule(self.checked_portal_type)
    result = module.newContent(portal_type=self.checked_portal_type)
    result.setProperty(self.int_catalogued_variable_id, self.int_value)
    assert result.getValidationState() == self.checked_validation_state
    return result

  def getWorklistDocumentCountFromActionName(self, action_name):
    self.assertEquals(action_name[-1], ')')
    left_parenthesis_offset = action_name.rfind('(')
    self.assertNotEquals(left_parenthesis_offset, -1)
    return int(action_name[left_parenthesis_offset + 1:-1])

  def associatePropertySheet(self):
    from Products.ERP5Type.Base import _aq_reset
    ti = self.getTypesTool().getTypeInfo(self.checked_portal_type)
    ti.property_sheet_list = list(ti.property_sheet_list) + \
                                ['SortIndex']
    # reset aq_dynamic cache
    _aq_reset()

  def addWorkflowCataloguedVariable(self):
    workflow = self.getWorkflowTool()[self.checked_workflow]
    workflow.variables.addVariable(self.int_catalogued_variable_id)
    variable = getattr(workflow.variables, self.int_catalogued_variable_id)

  def createWorklist(self):
    workflow = self.getWorkflowTool()[self.checked_workflow]
    worklists = workflow.worklists

    for worklist_id, actbox_name, role, expr, state, int_variable in [
          (self.worklist_assignor_id, self.actbox_assignor_name, 
           'Assignor', None, self.checked_validation_state, None),
          (self.worklist_owner_id, self.actbox_owner_name, 
           'Owner', None, self.checked_validation_state, None),
          (self.worklist_desactivated_id, self.actbox_desactivated_by_expression, 
           'Owner', 'python: 0', self.checked_validation_state, None),
          (self.worklist_wrong_state_id, self.actbox_wrong_state, 
           'Owner', None, self.not_checked_validation_state, None),
          (self.worklist_assignor_owner_id, self.actbox_assignor_owner_name, 
           'Assignor; Owner', None, self.checked_validation_state, None),
          (self.worklist_int_variable_id, self.actbox_int_variable_name, 
           None, None, None, str(self.int_value)),
    ]:
      worklists.addWorklist(worklist_id)
      worklist_definition = worklists._getOb(worklist_id)
      worklist_definition.setProperties('',
          actbox_name='%s (%%(count)s)' % (actbox_name, ),
          props={'guard_roles': role,
                 'var_match_portal_type': self.checked_portal_type,
                 'var_match_validation_state': state,
                 # Variable value is saved as string in worklist
                 'var_match_%s' % self.int_catalogued_variable_id: \
                                                            int_variable,
                  'guard_expr': expr})

  def clearCache(self):
    self.portal.portal_caches.clearAllCache()

  def test_01_worklist(self, quiet=0, run=run_all_test):
    """
    Test the permission of the building module.
    """
    if not run: 
      return

    workflow_tool = self.portal.portal_workflow

    self.logMessage("Create Manager")
    self.createManagerAndLogin()
    self.createUsers()
    self.logMessage("Create worklist")
    self.associatePropertySheet()
    self.addWorkflowCataloguedVariable()
    self.createWorklist()
    self.logMessage("Create document as Manager")
    document = self.createDocument()

    get_transaction().commit()
    self.tic()
    self.clearCache()

    result = workflow_tool.listActions(object=document)
    self.logout()

    # Users can not see worklist as they are not Assignor
    for user_id in ('manager', ):
      self.login(user_id)
      result = workflow_tool.listActions(object=document)
      self.logMessage("Check %s worklist as Assignor" % user_id)
      entry_list = [x for x in result \
                    if x['name'].startswith(self.actbox_assignor_name)]
      self.assertEquals(len(entry_list), 0)
      self.logMessage("Check %s worklist as Owner" % user_id)
      entry_list = [x for x in result \
                    if x['name'].startswith(self.actbox_owner_name)]
      self.assertEquals(len(entry_list), 1)
      self.assertEquals(
        self.getWorklistDocumentCountFromActionName(entry_list[0]['name']), 1)
      self.logout()
    for user_id in ('foo', 'bar'):
      self.logMessage("Check %s worklist" % user_id)
      self.login(user_id)
      result = workflow_tool.listActions(object=document)
      self.assertEquals(result, [])
      self.logout()

    # Define foo as Assignor
    self.login('manager')
    self.logMessage("Give foo Assignor role")
    document.manage_addLocalRoles('foo', ['Assignor'])
    self.logMessage("Give manager Assignor role")
    document.manage_addLocalRoles('manager', ['Assignor'])
    document.reindexObject()
    get_transaction().commit()
    self.tic()
    self.clearCache()
    self.logout()

    for user_id in ('manager', ):
      self.login(user_id)
      result = workflow_tool.listActions(object=document)
      self.logMessage("Check %s worklist as Assignor" % user_id)
      entry_list = [x for x in result \
                    if x['name'].startswith(self.actbox_assignor_name)]
      self.assertEquals(len(entry_list), 1)
      self.assertEquals(
        self.getWorklistDocumentCountFromActionName(entry_list[0]['name']), 1)
      self.logMessage("Check %s worklist as Owner" % user_id)
      entry_list = [x for x in result \
                    if x['name'].startswith(self.actbox_owner_name)]
      self.assertEquals(len(entry_list), 1)
      self.assertEquals(
        self.getWorklistDocumentCountFromActionName(entry_list[0]['name']), 1)
      self.logMessage("Check %s worklist as Owner and Assignor" % user_id)
      entry_list = [x for x in result \
                    if x['name'].startswith(self.actbox_assignor_owner_name)]
      self.assertEquals(len(entry_list), 1)
      self.assertEquals(
        self.getWorklistDocumentCountFromActionName(entry_list[0]['name']), 1)
      self.logout()
    for user_id in ('bar', ):
      self.login(user_id)
      result = workflow_tool.listActions(object=document)
      self.logMessage("Check %s worklist as Assignor" % user_id)
      entry_list = [x for x in result \
                    if x['name'].startswith(self.actbox_assignor_name)]
      self.assertEquals(len(entry_list), 0)
      self.logMessage("Check %s worklist as Owner" % user_id)
      entry_list = [x for x in result \
                    if x['name'].startswith(self.actbox_owner_name)]
      self.assertEquals(len(entry_list), 0)
      self.logMessage("Check %s worklist as Owner and Assignor" % user_id)
      entry_list = [x for x in result \
                    if x['name'].startswith(self.actbox_assignor_owner_name)]
      self.assertEquals(len(entry_list), 0)
      self.logout()
    for user_id in ('foo', ):
      self.login(user_id)
      result = workflow_tool.listActions(object=document)
      self.logMessage("Check %s worklist as Assignor" % user_id)
      entry_list = [x for x in result \
                    if x['name'].startswith(self.actbox_assignor_name)]
      self.assertEquals(len(entry_list), 1)
      self.assertTrue(
        self.getWorklistDocumentCountFromActionName(entry_list[0]['name']), 1)
      self.logMessage("Check %s worklist as Owner" % user_id)
      entry_list = [x for x in result \
                    if x['name'].startswith(self.actbox_owner_name)]
      self.assertEquals(len(entry_list), 0)
      self.logMessage("Check %s worklist as Owner and Assignor" % user_id)
      entry_list = [x for x in result \
                    if x['name'].startswith(self.actbox_assignor_owner_name)]
      self.assertEquals(len(entry_list), 1)
      self.assertEquals(
        self.getWorklistDocumentCountFromActionName(entry_list[0]['name']), 1)
      self.logout()

    # Define foo and bar as Assignee
    self.login('manager')
    self.logMessage("Give foo Assignee role")
    document.manage_addLocalRoles('foo', ['Assignee'])
    self.logMessage("Give bar Assignee role")
    document.manage_addLocalRoles('bar', ['Assignee'])
    document.reindexObject()
    get_transaction().commit()
    self.tic()
    self.clearCache()
    self.logout()

    # Users can not see worklist as they are not Assignor
    for user_id in ('manager', ):
      self.login(user_id)
      result = workflow_tool.listActions(object=document)
      self.logMessage("Check %s worklist as Assignor" % user_id)
      entry_list = [x for x in result \
                    if x['name'].startswith(self.actbox_assignor_name)]
      self.assertEquals(len(entry_list), 1)
      self.assertTrue(
        self.getWorklistDocumentCountFromActionName(entry_list[0]['name']), 1)
      self.logMessage("Check %s worklist as Owner" % user_id)
      entry_list = [x for x in result \
                    if x['name'].startswith(self.actbox_owner_name)]
      self.assertEquals(len(entry_list), 1)
      self.assertTrue(
        self.getWorklistDocumentCountFromActionName(entry_list[0]['name']), 1)
      self.logout()
    for user_id in ('bar', ):
      self.login(user_id)
      result = workflow_tool.listActions(object=document)
      self.logMessage("Check %s worklist as Assignor" % user_id)
      entry_list = [x for x in result \
                    if x['name'].startswith(self.actbox_assignor_name)]
      self.assertEquals(len(entry_list), 0)
      self.logMessage("Check %s worklist as Owner" % user_id)
      entry_list = [x for x in result \
                    if x['name'].startswith(self.actbox_owner_name)]
      self.assertEquals(len(entry_list), 0)
      self.logout()
    for user_id in ('foo', ):
      self.login(user_id)
      result = workflow_tool.listActions(object=document)
      self.logMessage("Check %s worklist as Assignor" % user_id)
      entry_list = [x for x in result \
                    if x['name'].startswith(self.actbox_assignor_name)]
      self.assertEquals(len(entry_list), 1)
      self.assertTrue(
        self.getWorklistDocumentCountFromActionName(entry_list[0]['name']), 1)
      self.logMessage("Check %s worklist as Owner" % user_id)
      entry_list = [x for x in result \
                    if x['name'].startswith(self.actbox_owner_name)]
      self.assertEquals(len(entry_list), 0)
      self.logout()

    # Check if int variable are managed by the worklist
    user_id = 'manager'
    self.login(user_id)
    result = workflow_tool.listActions(object=document)
    self.logMessage("Check %s worklist with int value as %s" % \
                                   (user_id, self.int_value))
    entry_list = [x for x in result \
                  if x['name'].startswith(self.actbox_int_variable_name)]
    self.assertEquals(len(entry_list), 1)

    # Change int value on document
    new_value = self.int_value + 1
    document.setProperty(self.int_catalogued_variable_id, new_value)
    get_transaction().commit()
    self.tic()
    self.clearCache()

    result = workflow_tool.listActions(object=document)
    self.logMessage("Check %s worklist with int value as %s" % \
                                   (user_id, new_value))
    entry_list = [x for x in result \
                  if x['name'].startswith(self.actbox_int_variable_name)]
    self.assertEquals(len(entry_list), 0)

    self.logout()

    #
    # Check monovalued security role
    #
    self.login('manager')
    module = self.getPortal().getDefaultModule(self.checked_portal_type)
    module.manage_setLocalRoles('bar', ['Author'])

    self.login('bar')

    bar_document = self.createDocument()
    bar_document.manage_permission('View', ['Owner', 'Assignee'], 0)

    bar_assignee_document = self.createDocument()
    bar_assignee_document.manage_setLocalRoles('manager', ['Assignee'])
    bar_assignee_document.manage_permission('View', ['Owner', 'Assignee'], 0)

    self.logout()
    self.login('manager')

    module.manage_delLocalRoles('bar')

    # User can not see worklist as user can not view the document
    document.manage_setLocalRoles('manager', ['Owner', 'Assignee'])
    document.manage_permission('View', [], 0)
    document.reindexObject()
    get_transaction().commit()
    self.tic()
    self.clearCache()
    self.logout()

    for user_id in ('manager', ):
      self.login(user_id)
      result = workflow_tool.listActions(object=document)
      self.logMessage("Check %s worklist as Owner" % user_id)
      entry_list = [x for x in result \
                    if x['name'].startswith(self.actbox_owner_name)]
      self.assertEquals(len(entry_list), 0)
      self.logout()

    # User can not see worklist as Owner can not view the document
    document.manage_setLocalRoles('manager', ['Owner', 'Assignee'])
    document.manage_permission('View', ['Assignee'], 0)
    document.reindexObject()
    get_transaction().commit()
    self.tic()
    self.clearCache()
    self.logout()

    for user_id in ('manager', ):
      self.login(user_id)
      result = workflow_tool.listActions(object=document)
      self.logMessage("Check %s worklist as Owner" % user_id)
      entry_list = [x for x in result \
                    if x['name'].startswith(self.actbox_owner_name)]
      self.assertEquals(len(entry_list), 0)
      self.logout()

    # User can see worklist as Owner can view the document
    document.manage_permission('View', ['Owner', 'Assignee'], 0)
    document.reindexObject()
    get_transaction().commit()
    self.tic()
    self.clearCache()
    self.logout()

    for user_id in ('manager', ):
      self.login(user_id)
      result = workflow_tool.listActions(object=document)
      self.logMessage("Check %s worklist as Owner" % user_id)
      entry_list = [x for x in result \
                    if x['name'].startswith(self.actbox_owner_name)]
      self.assertEquals(len(entry_list), 1)
      self.assertTrue(
        self.getWorklistDocumentCountFromActionName(entry_list[0]['name']), 1)
      self.logout()

    # Define a local role key
    sql_catalog = self.portal.portal_catalog.getSQLCatalog()
    current_sql_catalog_local_role_keys = \
          sql_catalog.sql_catalog_local_role_keys
    sql_catalog.sql_catalog_local_role_keys = ('Owner | owner', )
    get_transaction().commit()
    self.portal.portal_caches.clearAllCache()

    try:

      # User can not see worklist as user can not view the document
      document.manage_setLocalRoles('manager', ['Owner', 'Assignee'])
      document.manage_permission('View', [], 0)
      document.reindexObject()
      get_transaction().commit()
      self.tic()
      self.clearCache()
      self.logout()

      for user_id in ('manager', ):
        self.login(user_id)
        result = workflow_tool.listActions(object=document)
        self.logMessage("Check %s worklist as Owner" % user_id)
        entry_list = [x for x in result \
                      if x['name'].startswith(self.actbox_owner_name)]
        self.assertEquals(len(entry_list), 0)
        self.logout()

      # User can see worklist as Assignee can view the document
      document.manage_permission('View', ['Assignee'], 0)
      document.reindexObject()
      get_transaction().commit()
      self.tic()
      self.clearCache()
      self.logout()

      for user_id in ('manager', ):
        self.login(user_id)
        result = workflow_tool.listActions(object=document)
        self.logMessage("Check %s worklist as Owner" % user_id)
        entry_list = [x for x in result \
                      if x['name'].startswith(self.actbox_owner_name)]
        self.assertEquals(len(entry_list), 1)
        self.assertTrue(
          self.getWorklistDocumentCountFromActionName(entry_list[0]['name']), 1)
        self.logout()

      # User can see worklist as Owner can view the document
      document.manage_permission('View', ['Owner', 'Assignee'], 0)
      document.reindexObject()
      get_transaction().commit()
      self.tic()
      self.clearCache()
      self.logout()

      for user_id in ('manager', ):
        self.login(user_id)
        result = workflow_tool.listActions(object=document)
        self.logMessage("Check %s worklist as Owner" % user_id)
        entry_list = [x for x in result \
                      if x['name'].startswith(self.actbox_owner_name)]
        self.assertEquals(len(entry_list), 1)
        self.assertTrue(
          self.getWorklistDocumentCountFromActionName(entry_list[0]['name']), 1)
        self.logout()
    finally:
      sql_catalog.sql_catalog_local_role_keys = \
          current_sql_catalog_local_role_keys
      get_transaction().commit()

def test_suite():
  suite = unittest.TestSuite()
  suite.addTest(unittest.makeSuite(TestWorklist))
  return suite
